/*
 *  ===============================================================================
 *  TestVisitorInterface.java: Create a collection of simple and compound
 *  features and then visit them with variety of visitor models ....
 *
 *  Written by: Mark Austin                                              March 2016
 *  ===============================================================================
 */

package visitor03;

import java.util.*;
import java.lang.Math;

import visitor03.model.*;
import visitor03.model.primitive.*;
import visitor03.model.compound.*;

public class TestVisitorInterface {
   public static void main(String[] args) {

      // Part 01: Create and print primitive polygon objects ...

      System.out.println("");
      System.out.println("*** Part 01: Create polygon, car, house and tree objects");
      System.out.println("*** ====================================================");
      System.out.println("");

      // Create a generic polygon model ....

      double x[] = { 5.0, 30.0, 40.0, 40.0,  5.0 };
      double y[] = { 5.0,  0.0, 10.0, 40.0, 40.0 };
      Feature poly = new SimplePolygon( x, y );
      poly.setName ("Generic Polygon");

      // Create polygon for University of Maryland ...

      double x2[] = { 0.0, 30.0,  30.0,  60.0,  60.0, 190.0,  190.0,  0.0 };
      double y2[] = { 0.0,  0.0,  20.0,  20.0,   0.0,   0.0,   40.0, 40.0 };
      Feature umd = new SimplePolygon( x2, y2 );
      umd.setName ("University of Maryland");

      // Create car and house compound objects ...

      HouseModel house01 = new HouseModel( 100.0, 100.0, 20.0 );
      house01.setName("Big house");
      house01.setPrice( 100000.00 );
      CarModel car01 = new CarModel( 100.0, 80.0, 30.0 );
      car01.setName("Sports car");
      car01.setPrice(  50000.00 );

      // Create a small forest of trees ...

      Tree tree01 = new Tree( 10.0, 10.0 );
      tree01.setName("Big Fur");
      tree01.setPrice(  5000.00 );
      Tree tree02 = new Tree( 30.0, 20.0 );
      tree02.setName("Weeping Willow");
      tree02.setPrice(  1000.00 );
      Tree tree03 = new Tree( 30.0, 25.0 );
      tree03.setName("Canadian Maple");
      tree03.setPrice(  2000.00 );

      // Part 02: Assemble and print composite hierarchy workspace ...

      System.out.println("");
      System.out.println("*** Part 02: Assemble coordinate system hierarchies      ");
      System.out.println("*** =====================================================");
      System.out.println("");

      // Add polygons to working area coordinate system ...

      CompositeHierarchy workingarea = new CompositeHierarchy( 100.0, 50.0, 0.0 );
      workingarea.add ( poly );
      workingarea.add ( umd );

      // Add trees to forest coordinate system ...

      CompositeHierarchy forest = new CompositeHierarchy( 200.0, 200.0, Math.PI/2.0 );
      forest.setName("Backyard woods");

      forest.add( tree01 );
      forest.add( tree02 );
      forest.add( tree03 );

      CompositeHierarchy workspace = new CompositeHierarchy( 0.0, 0.0, 0.0 );
      workspace.setName("House and Gardens");
      workspace.add( house01 );
      workspace.add( car01 );
      workspace.add( forest );

      // Part 05: Create and print copy of workspace ....

      System.out.println("");
      System.out.println("*** Part 03: Create and print copy of workspace");
      System.out.println("*** ===========================================");
      System.out.println("");

      CompositeHierarchy copy = workspace.clone();

      // Part 06: Visit composite hierarchy and compute total cost ... 

      System.out.println("");
      System.out.println("*** Part 04: Economic view of composite hierarchy");
      System.out.println("*** =============================================");
      System.out.println("");

      CompositePriceVisitor cpv = new CompositePriceVisitor();
      copy.accept( cpv );

      System.out.println("=============================" );
      System.out.printf( "Total cost   = %10.2f\n", cpv.getTotalPrice() );
      System.out.println("=============================" );

      // Part 07: Create print view of composite hierarchy ... 

      System.out.println("");
      System.out.println("*** Part 05: Print view of composite hierarchy");
      System.out.println("*** ==========================================");
      System.out.println("");

      CompositePrintVisitor cpr = new CompositePrintVisitor();
      copy.accept( cpr );

      System.out.println("");
      System.out.println("*** Part 05: Find and move named item in composite hierarchy");
      System.out.println("*** ========================================================");
      System.out.println("");

      System.out.println("");
      System.out.println("*** Try to find Big Fur ... ");
      System.out.println("");

      CompositeFindVisitor cfv01 = new CompositeFindVisitor();
      cfv01.setFeatureName("Big Fur");
      workspace.accept( cfv01 );

      Feature item01 = cfv01.getFeature();
      if (item01 != null) {
         System.out.printf("*** Returned item: name = %s ... \n", item01.getName() );
         System.out.printf("*** Local (x,y) = (%8.2f, %8.2f) ... \n", item01.getX(), item01.getY() );
         System.out.println ( item01.toString() );
      }

      System.out.println("");
      System.out.println("*** Try to find Canadian Maple ... ");
      System.out.println("");

      CompositeFindVisitor cfv02 = new CompositeFindVisitor("Canadian Maple");
      workspace.accept( cfv02 );

      Feature item02 = cfv02.getFeature();
      if (item02 != null) {
         System.out.printf("*** Returned item: name = %s ... \n", item02.getName() );
         System.out.printf("*** Local (x,y) = (%8.2f, %8.2f) ... \n", item02.getX(), item02.getY() );
         System.out.println ( item02.toString() );
      }

      System.out.println("");
      System.out.println("*** Try to find Big house ... ");
      System.out.println("");

      CompositeFindVisitor cfv03 = new CompositeFindVisitor("Big house");
      workspace.accept( cfv03 );

      Feature item03 = cfv03.getFeature();
      if (item03 != null) {
         System.out.printf("*** Returned item: name = %s ... \n", item03.getName() );
         System.out.printf("*** Local (x,y) = (%8.2f, %8.2f) ... \n", item03.getX(), item03.getY() );
         System.out.println ( item03.toString() );
      }

      System.out.println("");
      System.out.println("*** Try to find Sports car ... ");
      System.out.println("");

      CompositeFindVisitor cfv04 = new CompositeFindVisitor("Sports car");
      workspace.accept( cfv04 );

      Feature item04 = cfv04.getFeature();
      if (item04 != null) {
         System.out.printf("*** Returned item: name = %s ... \n", item04.getName() );
         System.out.printf("*** Local (x,y) = (%8.2f, %8.2f) ... \n", item04.getX(), item04.getY() );
         System.out.println ( item04.toString() );
      }

      System.out.println("");
      System.out.println("*** ====================================================");
      System.out.println("*** Move the Big Fur tree, Big house and Sports car ... ");
      System.out.println("*** ====================================================");
      System.out.println("");

      if (item01 != null) {
         item01.setPosition( 20.0, 20.0 );
      }

      if (item03 != null) {
         item03.setPosition( 200.0, 200.0 );
      }

      if (item04 != null) {
         item04.setPosition( 200.0, 200.0 );
      }

      // ===============================================
      // Print new composite hierarchy ....
      // ===============================================

      workspace.accept( cpr );

   }
}
